"""
Copyright (c) 2025 FAST-HEP. All rights reserved.

fasthep-toolbench: Package for cross-package utility functions
"""

from __future__ import annotations

from ._version import version as __version__

__all__ = ["__version__"]
